{This file was created with help:
MTF_100A.PDF - Microsoft Tape Format
mtf.h - head-file from Microsoft Tape Format reader (mtf-0.2.1)
and other...
}
{===========================================================================
 Description:   Structures and constants used in writing MTF and OTC.
 Notes:         References are made throughout this header to a Logical
                Block Address (LBA).  The LBA of a given DBLK is its
                offset in Logical Blocks from the FIRST SSET of the set
                it is associated with.  The size of a Logical Block for
                a given tape is defined in the Tape Header.  The LBA is
                used along with the Physical Block Address (PBA) to
                calculate the physical location of a DBLK on tape.
                TAPE and EOTM DBLKs have no associated SSET, so the
                concept of an LBA is not defined for these blocks.
                The SSET DBLK itself is at LBA 0.  However, if a set
                crosses EOM and a continuation tape is written, the
                LBA in the continuation SSET will contain an adjusted
                logical offset, along with a PBA, to allow for the
                calculation of physical locations of DBLKs on the
                continuation tape.
===========================================================================}
const

 CmtfMinTapeBlockSize      = 512;
 CmtfMaxTapeBlockSize      = 65536;
 CmtfMaxPrintString        = 100;
 CmtfMaxPatternLen         = 100;
 CmtfMaxPattern            = 20;

 CmtfCaseSensitive         = 0;
 CmtfCaseLower             = 1;
 CmtfCaseUpper             = 2;


// values for TmtfDbHdr.type field
 Cmtf_TAPE = $45504154; // Tape Header
 Cmtf_SSET = $54455353; // Start of Backup Set
 Cmtf_VOLB = $424C4F56; // Volume Descriptor
 Cmtf_DIRB = $42524944; // Directory Descriptor
 Cmtf_FILE = $454C4946; // File Descriptor
 Cmtf_CFIL = $4C494643; // Corrupt File Descriptor
 Cmtf_ESPB = $42505345; // End of Set Pad Descriptor
 Cmtf_ESET = $54455345; // End of Backup Set
 Cmtf_EOTM = $4D544F45; // End of Tape
 Cmtf_SFMB = $424D4653;
 Cmtf_IMAG = $47414D49; // Image Descriptor       
 Cmtf_SSES = $53455353; // Start Of Session Descriptor
 Cmtf_ESES = $53455345; // End of Session Descriptor
 Cmtf_EOBJ = $4A424F45; // End of Object Descriptor

// platform-independant stream data types
 Cmtf_STAN = $4E415453; // standard
 Cmtf_PNAM = $4D414E50; // path
 Cmtf_FNAM = $4D414E46; // file name
 Cmtf_CSUM = $4D555343; // checksum
 Cmtf_CRPT = $54505243; // corrupt
 Cmtf_SPAD = $44415053; // pad
 Cmtf_SPAR = $52415053; // sparse
 Cmtf_TSMP = $504D5354; // set map, media based catalog, type 1
 Cmtf_TFDD = $44444654; // fdd, media based catalog, type 1
 Cmtf_MAP2 = $3250414D; // set map, media based catalog, type 2
 Cmtf_FDD2 = $32444446; // fdd, media based catalog, type 2
 Cmtf_SCRC = $43524353; // stream crc

// Windows NT stream data types
 Cmtf_ADAT = $54414441; // NTFS alternate data
 Cmtf_NTEA = $4145544E; // NTFS EA data
 Cmtf_NACL = $4C43414E; // NT security data
 Cmtf_NTED = $4445544E;
 Cmtf_NTQU = $5551544E;
 Cmtf_NTPR = $5250544E;
 Cmtf_NTOI = $494F544E;

// Windows 95 stream data types
 Cmtf_GERC = $43524547;

// Netware stream data types
 Cmtf_N286 = $3638324E;
 Cmtf_N386 = $3638334E;
 Cmtf_NBND = $444E424E;
 Cmtf_SMSD = $44534D53;

// OS/2 stream data types
 Cmtf_OACL = $4C43414F; // HPFS security data
 Cmtf_O2EA = $4145324F; // HPFS EA data

// Macintosh stream data types
 Cmtf_MRSC = $4353524D;
 Cmtf_MPRV = $5652504D;
 Cmtf_MINF = $464E494D;

// MSSQL stream data types
 Cmtf_MSCI = $4943534D;
 Cmtf_MQCI = $4943514D;
 Cmtf_MSDA = $4144534D;
 Cmtf_MQDA = $4144514D;
 Cmtf_MSTL = $4C54534D;
 Cmtf_MQTL = $4C54514D;
 Cmtf_MSLS = $534C534D;
 Cmtf_SCIN = $4E494353;    // SQL Config Info
 Cmtf_SFGI = $49474653;    // SQL File Group Info
 Cmtf_SFIN = $4E494653;    // SQL File Info
 Cmtf_APAD = $44415041;    // Alignment Pad
 Cmtf_MPAD = $4441504D;

// some type
 Cmtf_OTCP = $5043544f;

 Cmtf_CMP_HDR_ID = $4846;  // 'FH'

// bit masks for TmtfDbHdr.attr field for all values of TmtfDbHdr.type
 Cmtf_CONTINUATION = 1;
 Cmtf_COMPRESSION  = 2;
 Cmtf_EOS_AT_EOM   = 4;

// bit masks for TmtfDbHdr.attr field for TmtfDbHdr.type = MTF_TAPE
 Cmtf_SET_MAP_EXISTS = $100;
 Cmtf_FDD_ALLOWED    = $200;

// bit masks for TmtfDbHdr.attr field for TmtfDbHdr.type = MTF_SSET
 Cmtf_FDD_EXISTS = $100;
 Cmtf_ENCRYPTION = $200;

// bit masks for TmtfDbHdr.attr field for TmtfDbHdr.type = MTF_ESET
 Cmtf_FDD_ABORTED   = $100;
 Cmtf_END_OF_FAMILY = $200;
 Cmtf_ABORTED_SET   = $400;

// bit masks for TmtfDbHdr.attr field for TmtfDbHdr.type = MTF_EOTM
 Cmtf_NO_ESET_PBA      = $100;
 Cmtf_INVALID_ESET_PBA = $200;

// values for TmtfDbHdr.osId field
 Cmtf_OS_NETWARE     = 1;
 Cmtf_OS_NETWARE_SMS = 13;
 Cmtf_OS_WINDOWS_NT  = 14;
 Cmtf_OS_DOS         = 24;
 Cmtf_OS_OS2         = 25;
 Cmtf_OS_WINDOWS_95  = 26;
 Cmtf_OS_MACINTOSH   = 27;
 Cmtf_OS_UNIX        = 28;

// values for TmtfDbHdr.strType field
 Cmtf_NO_STRINGS  = 0;
 Cmtf_ANSI_STR    = 1;
 Cmtf_UNICODE_STR = 2;

// bitmasks for MTF_TAPE_BLK.attr
 Cmtf_TAPE_SOFT_FILEMARK_BIT = 1;
 Cmtf_TAPE_MEDIA_LABEL_BIT   = 2;

// values for MTF_TAPE_BLK.catType
 Cmtf_NO_MBC     = 0;
 Cmtf_MBC_TYPE_1 = 1;
 Cmtf_MBC_TYPE_2 = 2;

// bitmasks for MTF_SSET_BLK.attr
 Cmtf_SSET_TRANSFER_BIT     = 1;
 Cmtf_SSET_COPY_BIT         = 2;
 Cmtf_SSET_NORMAL_BIT       = 4;
 Cmtf_SSET_DIFFERENTIAL_BIT = 8;
 Cmtf_SSET_INCREMENTAL_BIT  = $10;
 Cmtf_SSET_DAILY_BIT        = $20;

// value for MTF_SSET_BLK.tz when local time is not coordinated with UTC
 Cmtf_LOCAL_TZ = 127;

// bitmasks for MTF_VOLB_BLK.attr
 Cmtf_VOLB_NO_REDIRECT_RESTORE_BIT = 1;
 Cmtf_VOLB_NON_VOLUME_BIT          = 2;
 Cmtf_VOLB_DEV_DRIVE_BIT           = 4;
 Cmtf_VOLB_DEV_UNC_BIT             = 8;
 Cmtf_VOLB_DEV_OS_SPEC_BIT         = $10;
 Cmtf_VOLB_DEV_VEND_SPEC_BIT       = $20;

// bitmasks for MTF_DIRB_BLK.attr
 Cmtf_DIRB_READ_ONLY_BIT     = $100;
 Cmtf_DIRB_HIDDEN_BIT        = $200;
 Cmtf_DIRB_SYSTEM_BIT        = $400;
 Cmtf_DIRB_MODIFIED_BIT      = $800;
 Cmtf_DIRB_EMPTY_BIT         = $10000;
 Cmtf_DIR_PATH_IN_STREAM_BIT = $20000;
 Cmtf_DIRB_CORRUPT_BIT       = $40000;

// bitmasks for MTF_FILE_BLK.attr
 Cmtf_FILE_READ_ONLY_BIT      = $100;
 Cmtf_FILE_HIDDEN_BIT         = $200;
 Cmtf_FILE_SYSTEM_BIT         = $400;
 Cmtf_FILE_MODIFIED_BIT       = $800;
 Cmtf_FILE_IN_USE_BIT         = $10000;
 Cmtf_FILE_NAME_IN_STREAM_BIT = $20000;
 Cmtf_FILE_CORRUPT_BIT        = $40000;

// bitmasks for MTF_CFIL_BLK.attr
 Cmtf_CFIL_LENGTH_CHANGE_BIT  = $10000;
 Cmtf_CFIL_UNREADABLE_BLK_BIT = $20000;
 Cmtf_CFIL_DEADLOCK_BIT       = $80000;
 //Cmtf_FILE_READ_ONLY_BIT      = $100;

// bitmasks for MTF_ESET_BLK.attr
 Cmtf_ESET_TRANSFER_BIT       = 1;
 Cmtf_ESET_COPY_BIT           = 2;
 Cmtf_ESET_NORMAL_BIT         = 4;
 Cmtf_ESET_DIFFERENTIAL_BIT   = 8;
 Cmtf_ESET_INCREMENTAL_BIT    = $10;
 Cmtf_ESET_DAILY_BIT          = $20;

// bitmasks for MTF_STREAM_HDR.sysAttr
 Cmtf_STREAM_MODIFIED_FOR_READ = 1;
 Cmtf_STREAM_CONTAINS_SECURITY = 2;
 Cmtf_STREAM_IS_NON_PORTABLE   = 4;
 Cmtf_STREAM_IS_SPARSE         = 8;

// bitmasks for MTF_STREAM_HDR.mediaAttr
 Cmtf_STREAM_CONTINUE          = 1;
 Cmtf_STREAM_VARIABLE          = 2;
 Cmtf_STREAM_VAR_END           = 4;
 Cmtf_STREAM_ENCRYPTED         = 8;
 Cmtf_STREAM_COMPRESSED        = $10;
 Cmtf_STREAM_CHECKSUMED        = $20;
 Cmtf_STREAM_EMBEDDED_LENGTH   = $40;

type

 {GUID = array[0..15]of byte;}

 {pqword = ^qword;
 qword = packed record
  _least:dword;                // least significant 32 bits
  _most:dword;                 // most significant 32 bits
 end;}

// pointer to non-fixed length information
 PmtfTapeAddress = ^TmtfTapeAddress;
 TmtfTapeAddress = packed record
  _size:word;                  // size of referenced field
  _off:word;                   // offset to start of field from start of structure
 end;

// storage of date and time
 PmtfDateTime = ^TmtfDateTime;      // 14 bit year, 4 bit month, 5 bit day,
 TmtfDateTime = array[0..4]of byte; // 5 bit hour, 6 bit minute, 6 bit second

// common descriptor block header
 PmtfDbHdr = ^TmtfDbHdr;
 TmtfDbHdr = packed record
  _type:dword;                 // DBLK type                "m_dblk_type"             
  _attr:dword;                 // block attributes         "m_attributes"            
  _off:word;                   // offset to first event    "m_first_event_offset"    
  _osId:byte;                  // OS ID                    "m_os_id"                 
  _osVer:byte;                 // OS version               "m_os_version"            
  _size:Int64;                 // displayable size         "m_displayable_size"      
  _fla:Int64;                  // format logical address   "m_format_logical_address"
  _mbc:word;                   // reserved for MBC         "m_reserved_for_mbc"      
  _rsv1:array[0..5]of byte;    // reserved for future use  
  _cbId:dword;                 // control block ID         "m_control_block_id"      
  _rsv2:array[0..3]of byte;    // reserved for future use  
  _osData:TmtfTapeAddress;     // OS-specific data         "m_os_specific_data"      
  _strType:byte;               // string type              "m_string_type"           
  _rsv3:byte;                  // reserved for future use  
  _check:word;                 // header checksum          "m_header_checksum"       
 end;

// structure pointed to by the TmtfDbHdr.osData field when
// TmtfDbHdr.osId = MTF_OS_WINDOWS_NT and TmtfDbHdr.osVer = 0
 PmtfOsDataWindowsNt = ^TmtfOsDataWindowsNt;
 TmtfOsDataWindowsNt = packed record
  _attr:dword;                 // file attributes
  _off:word;                   // short name offset
  _size:word;                  // short name size
  _link:ByteBool;              // if non-zero the file is a link to a previous file
  _rsv:word;                   // reserved for future use
 end;

// descriptor block for TmtfDbHdr.type = MTF_TAPE (tape header)
 PmtfTapeBlk = ^TmtfTapeBlk;
 TmtfTapeBlk = packed record
  _common:TmtfDbHdr;           // common block header
  _famId:dword;                // media family ID
  _attr:dword;                 // TAPE attributes
  _seq:word;                   // media sequence number
  _encrypt:word;               // password encryption
  _sfmSize:word;               // soft filemark block size
  _catType:word;               // media-based catalog type
  _name:TmtfTapeAddress;       // media name
  _desc:TmtfTapeAddress;       // media desc./label
  _passwd:TmtfTapeAddress;     // media password
  _software:TmtfTapeAddress;   // software name
  _flbSize:word;               // format logical block size
  _vendorId:word;              // software vendor ID
  _date:TmtfDateTime;          // media date
  _ver:byte;                   // MTF major version
 end;

// descriptor block for TmtfDbHdr.type = MTF_SSET (start of data set)
 PmtfSsetBlk = ^TmtfSsetBlk;
 TmtfSsetBlk = packed record
  _common:TmtfDbHdr;           // common block header
  _attr:dword;                 // SSET attributes
  _passEncrypt:word;           // password encryption
  _softCompress:word;          // software compression
  _vendor:word;                // software vendor ID
  _num:word;                   // data set number
  _name:TmtfTapeAddress;       // data set name
  _desc:TmtfTapeAddress;       // data set description
  _passwd:TmtfTapeAddress;     // data set password
  _user:TmtfTapeAddress;       // user name
  _pba:Int64;                  // physical block address
  _date:TmtfDateTime;          // media write date
  _major:byte;                 // software major version
  _minor:byte;                 // software minor version
  _tz:byte;                    // time zone (signed ?)
  _ver:byte;                   // MTF minor version
  _catVer:byte;                // media catalog version
 end;

// descriptor block for TmtfDbHdr.type = MTF_VOLB (volume)
 PmtfVolbBlk = ^TmtfVolbBlk;
 TmtfVolbBlk = packed record
  _common:TmtfDbHdr;           // common block header
  _attr:dword;                 // VOLB attributes
  _device:TmtfTapeAddress;     // device name
  _volume:TmtfTapeAddress;     // volume name
  _machine:TmtfTapeAddress;    // machine name
  _date:TmtfDateTime;          // media write date
 end;

// descriptor block for TmtfDbHdr.type = MTF_DIRB (directory)
 PmtfDirbBlk = ^TmtfDirbBlk;
 TmtfDirbBlk = packed record
  _common:TmtfDbHdr;           // common block header
  _attr:dword;                 // DIRB attributes
  _mod:TmtfDateTime;           // last modification date
  _create:TmtfDateTime;        // creation date
  _backup:TmtfDateTime;        // backup date
  _access:TmtfDateTime;        // last access date
  _id:dword;                   // directory ID
  _name:TmtfTapeAddress;       // directory name
 end;

// descriptor block for TmtfDbHdr.type = MTF_FILE (file)
 PmtfFileBlk = ^TmtfFileBlk;
 TmtfFileBlk = packed record
  _common:TmtfDbHdr;           // common block header
  _attr:dword;                 // FILE attributes
  _mod:TmtfDateTime;           // last modification date
  _create:TmtfDateTime;        // creation date
  _backup:TmtfDateTime;        // backup date
  _access:TmtfDateTime;        // last access date
  _dirId:dword;                // directory ID
  _id:dword;                   // file ID
  _name:TmtfTapeAddress;       // file name
 end;

// descriptor block for TmtfDbHdr.type = MTF_CFIL (corrupt object)
 PmtfCfilBlk = ^TmtfCfilBlk;
 TmtfCfilBlk = packed record
  _common:TmtfDbHdr;           // common block header
  _attr:dword;                 // CFIL attributes
  _rsv:array[0..7]of byte;     // reserved for future use
  _off:Int64;                  // stream offset
  _num:Int64;                  // corrupt stream number
 end;

// descriptor block for TmtfDbHdr.type = MTF_ESPB (end of set pad)
 PmtfEspbBlk = ^TmtfEspbBlk;
 TmtfEspbBlk = TmtfDbHdr;

// descriptor block for TmtfDbHdr.type = MTF_ESET (end of data set)
 PmtfEsetBlk = ^TmtfEsetBlk;
 TmtfEsetBlk = packed record
  _common:TmtfDbHdr;           // common block header
  _attr:dword;                 // ESET attributes           "m_eset_attributes"          
  _corrupt:dword;              // number of corrupt files   "m_corrupt_file_count"       
  _mbc1:Int64;                 // reserved for MBC          "m_set_map_pba"              
  _mbc2:Int64;                 // reserved for MBC          "m_file_detail_pba"          
  _seq:word;                   // FDD media sequence number "m_fdd_media_sequence_number"
  _set:word;                   // data set number           "m_data_set_number"          
  _date:TmtfDateTime;          // media write date          "m_media_write_date"         
 end;

// descriptor block for TmtfDbHdr.type = MTF_EOTM (end of tape)
 PmtfEotmBlk = ^TmtfEotmBlk;
 TmtfEotmBlk = packed record
  _common:TmtfDbHdr;           // common block header
  _lastEset:Int64;             // last ESET PBA         "m_last_eset_pba"
 end;

// descriptor block for TmtfDbHdr.type = MTF_SFMB (soft filemark)
 PmtfSfmbBlk = ^TmtfSfmbBlk;
 TmtfSfmbBlk = packed record
  _common:TmtfDbHdr;           // common block header
  _marks:dword;                // number of filemark entries
  _used:dword;                 // filemark entries used
 end;

// stream header
 PmtfStreamHdr = ^TmtfStreamHdr;
 TmtfStreamHdr = packed record        //                                "MTF_STREAM_HDR"
  _id:dword;                   // stream ID                      "m_id"
  _sysAttr:word;               // stream file system attributes  "m_file_system_attributes"
  _mediaAttr:word;             // stream media format attributes "m_media_format_attributes"
  _length:Int64;               // stream length                  "m_length"
  _encrypt:word;               // data encryption algorithm      "m_data_encryption_algorithm"
  _compress:word;              // data compression algorithm     "m_data_compression_algorithm"
  _check:word;                 // checksum                       "m_header_checksum"
 end;

// stream compression frame header
 PmtfCmpHdr = ^TmtfCmpHdr;
 TmtfCmpHdr = packed record
  _id:word;                    // compression header id  - see define below
  _attr:word;                  // stream media format attributes
  _remain:Int64;               // remaining stream size
  _uncompress:dword;           // uncompressed size
  _compress:dword;             // compressed size
  _seq:byte;                   // sequence number
  _rsv:byte;                   // reserved
  _check:word;                 // checksum
 end;

// Image DBLK Structure (IMAG)
 PmtfImagBlk = ^TmtfImagBlk;
 TmtfImagBlk = packed record
  _common:TmtfDbHdr;           // Common header
  _attr:dword;                 // External Attributes
  _size:dword;                 // Size of partition in bytes
  _bps:dword;                  // Number of bytes per sector
  _nsec:dword;                 // Number of sectors
  _rel:dword;                  // Relative sector number
  _partno:dword;               // Partition number of sector
  _ind:word;                   // Partition system indicator
  _heads:word;                 // Number of heads
  _name:TmtfTapeAddress;       // Offset/Size of partition name
 end;

// Start Of Session DBLK Structure (SSES)
 PmtfSsesBlk = ^TmtfSsesBlk;
 TmtfSsesBlk = packed record
  _common:TmtfDbHdr;           // Common header
  _lba:Int64;                   // Logical Block Address of this DBLK
 end;

// End Of Session DBLK Structure (ESES)
 PmtfEsesBlk = ^TmtfEsesBlk;
 TmtfEsesBlk = packed record
  _common:TmtfDbHdr;           // Common header
  _lba:Int64;                  // Logical Block Address of the SSES DBLK
                               //  that marks the start of this session
 end;

// End Of Object DBLK Structure (EOBJ)
 PmtfEobjBlk = ^TmtfEobjBlk;
 TmtfEobjBlk = packed record
  _common:TmtfDbHdr;           // Common header
  _lba:Int64;                  // Logical Block Address of the SSES DBLK
                               //  that marks the start of this session
  _obj:Int64;                  // Logical Block Address of the DBLK
                               //  that marks the start of the object just
                               //  backed up in this session
 end;

// On Tape Catalog (OTC) Structures

// Set Map Header Structure
 PmtfSmHdr = ^TmtfSmHdr;
 TmtfSmHdr = packed record
  _id:dword;                     // Unique tape family ID
  _num:word;                     // Number of Set Map entries to follow
  _pad:array[0..1]of byte;       // Ensure 4 byte alignment of first SM entry
 end;

// Set Map Entry Structure
 PmtfSmBlk = ^TmtfSmBlk;
 TmtfSmBlk = packed record
  _len:word;                     // Length of entry (including strings) in bytes
  _seq:word;                     // Tape number where set begins
  _attr:dword;                   // Tape Format Attributes
  _eattr:dword;                  // External Attributes
  _sset:Int64;                   // Address of associated SSET
  _fdd:Int64;                    // Address of associated F/DD
  _fdd_seq:word;                 // Tape number where F/DD begins
  _num:word;                     // Backup Set Number
  _lba:Int64;                    // See note at top of file
  _dirs:dword;                   // Number of directories in set
  _files:dword;                  // Number of files in set
  _corrupt:dword;                // Number of corrupt files in set
  _dsize:Int64;                  // Displayable size of assoc. SSET
  _volumes:word;                 // No. of OTC vol entries to follow
  _enalg:word;                   // Password Encryption Algorithm ID
  _name:TmtfTapeAddress;         // Offset/Size of set name string
  _passwd:TmtfTapeAddress;       // Offset/Size of description string
  _desc:TmtfTapeAddress;         // Offset/Size of password string
  _user:TmtfTapeAddress;         // Offset/Size of user name string
  _date:TmtfDateTime;           // Date/Time backup was started
  _zn:byte;                      // Time zone where backed up (signed ?)
  _osId:byte;                    // Machine/OS id where written
  _osVer:byte;                   // Machine/OS version where written
  _strType:byte;                 // Type of strings stored
  _formatVer:byte;               // tape_format_version_minor
  _catVer:byte;                  // tape_catalog_version
 end;

// Common File/Directory Detail Entry Header Structure
 PmtfFddHdr = ^TmtfFddHdr;
 TmtfFddHdr = packed record
  _len:dword;                    // Length of entry (including strings) in bytes
  _type:dword;                   // As in associated DBLK
  _seq:word;                     // Tape number of associated DBLK
  _attr:dword;                   // Tape Format Attributes
  _lba:Int64;                    // See note at top of file
  _dsize:Int64;                  // Displayable size of assoc. DBLK
  _link:word;
  _osId:byte;                    // Machine/OS id where written
  _osVer:byte;                   // Machine/OS version where written
  _strType:byte;                 // Type of strings stored
  _pad:byte;                     // Pad to alignment boundary
 end;

// File/Directory Detail Volume Entry Structure
 PmtfFddVol = ^TmtfFddVol;
 TmtfFddVol = packed record
  _attr:dword;                   // volb_attribs of associated VOLB
  _device:TmtfTapeAddress;       // Offset/Size of device name string
  _volume:TmtfTapeAddress;       // Offset/Size of volume name string
  _machine:TmtfTapeAddress;      // Offset/Size of machine name
  _date:TmtfDateTime;            // Last Backup Date (before this)
 end;

// File/Directory Detail Directory Entry Structure
 PmtfFddDir = ^TmtfFddDir;
 TmtfFddDir = packed record
  _modify:TmtfDateTime;          // Last Modified Date
  _create:TmtfDateTime;          // Date created
  _lastbackup:TmtfDateTime;      // Last Backup Date (before this)
  _access:TmtfDateTime;          // Last Access Date
  _attr:dword;                   // dirb_attribs of associated DIRB
  _name:TmtfTapeAddress;         // Offset/Size of path string
 end;

// File/Directory Detail File Entry Structure
 PmtfFddFile = ^TmtfFddFile;
 TmtfFddFile = packed record
  _modify:TmtfDateTime;          // Last Modified Date               
  _create:TmtfDateTime;          // Date created                     
  _lastbackup:TmtfDateTime;      // Last Backup Date (before this)   
  _access:TmtfDateTime;          // Last Access Date                 
  _attr:dword;                   // file_attribs of associated FILE  
  _name:TmtfTapeAddress;         // Offset/Size of file name string  
 end;

//=================================================================//

 PmtfLsn = ^TmtfLsn;
 TmtfLsn = packed record
   L1:dword;
   L2:dword;
   L3:word;
 end;

// RAID Extension
 PmtfRaidExt = ^TmtfRaidExt;
 TmtfRaidExt = packed record
  _id:GUID;                      // "m_family_id"
  _raidType:dword;               // "m_raid_type"
  _cnt:dword;                    // "m_device_count"
  _seq:dword;                    // "m_sequence_number"
  _strip:dword;                  // "m_striping_size"
 end;

// OS Info
 PmtfOsInfo = ^TmtfOsInfo;
 TmtfOsInfo = packed record
  m_binding_id:GUID;
  m_expiration_time:TmtfDateTime;
 end;

// common sql block header
 PmtfSqlHdr = ^TmtfSqlHdr;
 TmtfSqlHdr = packed record
  _type:dword;                   // "m_type"
  _size:dword;                   // "m_size"
 end;

// MSTL - MSSQL Tape Log ??
 PmtfMstlBlk = ^TmtfMstlBlk;
 TmtfMstlBlk = packed record
  _common:TmtfDbHdr;
  _attr:dword;                   // "m_mstl_attributes"
  _size:dword;                   // "m_io_size"
  _startOffs:Int64;              // "m_starting_offset"
  _endOffs:Int64;                // "m_ending_offset"
  _fnum:dword;                   // "m_file_number"
  _logHdr:TmtfTapeAddress;       // "m_log_header"
  _create:TmtfDateTime;          // "m_file_creation_time"
 end;

// MSCI - MSSQL Class Info ??
 PmtfMsciBlk = ^TmtfMsciBlk;
 TmtfMsciBlk = packed record
  _common:TmtfDbHdr;
  _attr:dword;                   // "m_msci_attributes"
 end;

// MSDA ??
 PmtfMsdaBlk = ^TmtfMsdaBlk;
 TmtfMsdaBlk =packed record
  _common:TmtfDbHdr;
  _attr:dword;                   // "m_msda_attributes"
  _size:dword;                   // "m_io_size"
 end;

// MSLS ??
 PmtfMslsBlk = ^TmtfMslsBlk;
 TmtfMslsBlk = packed record
  _common:TmtfDbHdr;
  _attr:dword;                   // "m_msls_attributes"
 end;

// SFGI - MSSQL File Group Info
 PmtfSfgiBlk = ^TmtfSfgiBlk;
 TmtfSfgiBlk = packed record
  _sh:TmtfSqlHdr;
  _status:dword;                 // "m_status"
  _id:word;                      // "m_group_id"
  _pol:word;                     // "m_allocation_policy"
  _name:TmtfTapeAddress;         // "m_group_name"
 end;

// SFIN - MSSQL File Info
 PmtfSfinBlk = ^TmtfSfinBlk;
 TmtfSfinBlk = packed record
  _sh:TmtfSqlHdr;
  _flag:dword;                   // 08: "m_flags"
  _headerSize:dword;             // 0C: "m_file_header_size"
  _pageSize:dword;               // 10: "m_page_size"
  _fnum:dword;                   // 14: "m_file_number"
  _pageCnt:dword;                // 18: "m_backed_up_page_count"
  _ftype:dword;                  // 1C: "m_file_type"
  _blockSize:dword;              // 20: "m_source_file_block_size"
  _grpId:word;                   // 24: "m_group_id"
  field_26:word;                 // 26: align ?
  _fsize:Int64;                  // 28: "m_file_size"
  _lname:TmtfTapeAddress;        // 30: "m_logical_name
  _pname:TmtfTapeAddress;        // 34: "m_physical_name"
  _lsn1:TmtfLsn;                 // 38: "m_first_create_index_lsn"
  _lsn2:TmtfLsn;                 // 42: "m_first_nonlogged_lsn"
  _lsn3:TmtfLsn;                 // 4C: "m_first_update_lsn
  _fMaxSize:Int64;               // 56: "m_file_max_size"
  _fileGrowth:Int64;             // 5E: "m_file_growth"
  _lsn4:TmtfLsn;                 // 66: "m_backup_lsn"
  field_70:word;                 // 70: ??
  _fMinSize:Int64;               // 72: "m_file_min_size"
  _uShrSize:Int64;               // 7A: "m_user_shrink_size"
  _aShrSize:Int64;               // 82: "m_actual_shrink_size"
// v.8.0
  _lsn5:TmtfLsn;                 // 8C: "m_create_lsn"
  _lsn6:TmtfLsn;                 // "m_drop_lsn"
  _lsn7:TmtfLsn;                 // "m_differential_base_lsn"
  _guid:GUID;                    // "m_differential_base_guid"
 end;

// SCIN - MSSQL Config Info
 PmtfScinBlk = ^TmtfScinBlk;
 TmtfScinBlk = packed record
  _sh:TmtfSqlHdr;
  _flags:dword;                  // 08: "m_flags"
  _dbCount:dword;                // 0C: "m_database_file_count"
  _logCount:dword;               // 10: "m_log_file_count"
  _nextId:dword;                 // 14: "m_next_id"
  _srvVer:dword;                 // 18: "m_server_version"
  _proc:dword;                   // 1C: "m_processor"
  _firstLsn:TmtfLsn;             // 20: "m_first_lsn"
  _lastLsn:TmtfLsn;              // 2A: "m_last_lsn"
  _checkpointLsn:TmtfLsn;        // 34: "m_checkpoint_lsn"
  _backupLsn:TmtfLsn;            // 3E: "m_backup_lsn"
  _creationDate:TmtfDateTime;    // 48: "m_creation_date"
  _backupStart:TmtfDateTime;     // 4D: "m_backup_start"
  _backupFinish:TmtfDateTime;    // 52: "m_backup_finish"
  _backupType:byte;              // 57: "m_backup_type"
                                 //  44h   = 1 (full backup)
                                 //  46h   = 4
                                 //  49h   = 5 (differential backup)
                                 //  4Ch   = 2
                                 //  other = 0 (unknown type)
                                 //
  _sortOrder:byte;               // 58: "m_sort_order
  _sortOrderStatus:byte;         // 59: "m_sort_order_status"
  _codePage:byte;                // 5A: "m_code_page"
  _compatLevel:byte;             // 5B: "m_compatibility_level"
  _dbId:word;                    // 5C: "m_database_id"
  _dbVer:word;                   // 5E: "m_database_version"
  _backupSize:Int64;             // 60: "m_backup_size"
  _dbName:TmtfTapeAddress;       // 68: "m_database_name
  _srvName:TmtfTapeAddress;      // 6C: "m_server_name
  _minRestartPage:dword;         // 70: "m_minimum_restart_page
  _uniLocale:dword;              // 74: "m_unicode_locale"
                                 //    if BasicSize < 78h = <null>
  _uniCmpFlags:dword;            // 78: "m_unicode_compare_flags"
                                 //    if BasicSize < 78h = <null>
                                 //    if BasicSize < 7Ch = 0
  _bindId:GUID;                  // 7C: "m_bindingId""
                                 //    if BasicSize < 8Ch = NULL_GUID
  _fileGeneration:dword;         // 8C: "m_file_generation"
// v.8.0
// "m_differential_base_guid"
// "m_recovery_fork_name_stack"
// "m_collation_id"
 end;

// MQDA, APAD, MPAD - TmtfStreamHdr
